@@ -5,6 +5,7 @@
 #'   data" also for those functions that cannot cope with labelled data by default.
 #'
 #' @param x A data frame.
+#' @inheritParams as_factor
 #'
 #' @return \code{x} with variable labels as column names. For variables without
 #'   variable labels, the column name is left unchanged.
@@ -24,7 +25,16 @@
 #' colnames(label_to_colnames(iris))
 #' plot(label_to_colnames(iris))
 #' @export
-label_to_colnames <- function(x) {
-  colnames(x) <- get_label(x, def.value = colnames(x))
+label_to_colnames <- function(x, ...) {
+  dots <- as.character(match.call(expand.dots = FALSE)$`...`)
+  .dat <- .get_dot_data(x, dots)
+
+  if (!is.null(ncol(.dat)) && ncol(.dat) > 0) {
+    replace_index <- match(colnames(.dat), colnames(x))
+    colnames(x)[replace_index] <- get_label(.dat, def.value = colnames(.dat))
+  } else {
+    colnames(x) <- get_label(x, def.value = colnames(x))
+  }
+
   x
 }
\ No newline at end of file
